;/****************************************************************
;*   Utility routines for C                                      *
;*****************************************************************
;*   PC Speaker Digital Audio Driver                             *
;*                                                               *
;*                                     Yudong Yang               *
;*      Dept. of Computer Science and Technology, TsingHua Univ. *
;*   File : adda386.asm                                          *
;****************************************************************/
; for Watcom C/386 flat small mode register calling code
;************************************************************
; FUNCTIONS SUPPLIED IN THIS PACKAGE
;************************************************************

public   talking_        ; send data to PC speaker
public  _talking         ; send data to PC speaker
public   talking2_       ; send data to PC speaker and double the sample rate
public  _talking2        ; send data to PC speaker and double the sample rate
public   cpuspeedtest_   ; test cpu speed
public  _cpuspeedtest    ; test cpu speed
public   setspeakerdata_ ; calculate PWM data for D/A using PC speaker and sample loop
public  _setspeakerdata  ; calculate PWM data for D/A using PC speaker and sample loop


comment #
/**************************************************************
*  C calling function interface                               *
**************************************************************/
void talking(unsigned char * sampledata, int samplelength);
void talking2(unsigned char * sampledata, int samplelength);
int cpuspeedtest(void);
void setspeakerdata(int samplerate);
/**************************************************************/
#

;!!!!!!!!!!!!!
.386p

;************************************************************
; PORT DEFINATIONS
;************************************************************

PortB   equ     61h             ; 8255 PortB
ComReg  equ     43h             ; 8253 command register
Latch2  equ     42h             ; 8253 timer 2 latch

CPU_LOOPS  equ   50000          ; run 50000 loops 
LeastValue equ  10

;************************************************************
; DEVICE DELAY MACRO DEFINATION
;************************************************************

DeviceDelay  macro      ;Delay a bit to allow time for device to respond
	jmp $+2         ; to successive I/O accesses
	jmp $+2
	jmp $+2
	endm

_DATA	SEGMENT DWORD PUBLIC 'DATA'
 _adda_machinespeed    dd   10   
 _adda_sample_rate     dd   11025
 _adda_PMWtable        dd   256 dup(0)              ; asummed 256 level
 _adda_int_mask        db   0, 0, 0, 0  
 _adda_windows_run     db   0, 0, 0, 0  
_DATA	ENDS
                                                
_TEXT	SEGMENT BYTE PUBLIC 'CODE'
	ASSUME	cs:_TEXT
        assume  DS:_DATA



IntCtl_Mask   equ   21h

close_timer_int proc near
	cli
	cmp   _adda_windows_run, 0
	;cmp   cs:_adda_windows_run, 0
	jnz   close_int
	push  ax
	in    al, IntCtl_Mask
	mov   _adda_int_mask, al
	;mov   cs:_adda_int_mask, al
	or    al, 1
	out   IntCtl_Mask, al
	pop   ax
	sti
close_int:      
	ret
close_timer_int endp

open_timer_int  proc near
	cmp   _adda_windows_run, 0
	;cmp   cs:_adda_windows_run, 0
	jnz   open_int
	cli
	push  ax
	mov   al, _adda_int_mask
	;mov   al, cs:_adda_int_mask
	out   IntCtl_Mask, al
	pop   ax
open_int:      
	sti
	ret
open_timer_int  endp


talking_        proc    near
_talking        proc    near
;	bufferptr       equ     dword ptr [bp+8]
;	smplength       equ     dword ptr [bp+12]
;       EAX now is bufferptr
;       EDX now is smplength
;	push    ebp
;	mov     ebp, esp

	cmp     edx, 0
	jbe     talking_rtn
	pushad
	pushfd                           ; to keep the flags
        
        push    eax
        push    edx
	call    close_timer_int

	
	in      al, PortB
	and     al, 11111100b
	mov     dh, al
	or	al, 00000010b
	mov	dl, al
	and     al, 11111100b
	DeviceDelay
	out     PortB, al
	
	mov     ebp, _adda_machinespeed
	;mov     ebp, cs:_adda_machinespeed

	cld
	pop     edi    ; smplength
	pop     esi    ; bufferptr

getnewbyte:
	lodsb

	xor     ah, ah
	cwde
	mov     ebx, _adda_PMWtable[eax*4]
	;mov     ebx, cs:_adda_PMWtable[eax*4]

loophigh:
	
	or      ebx, ebx
	jz      looplow
	
	mov     al, dh
	out     PortB, al
	
	mov     ecx, ebx
delayhigh:      loop delayhigh

looplow:
	cmp     ebx, ebp
	jnb     restore_speaker
	
	mov     al, dl
	out     PortB, al
	
	mov     ecx, ebp
	sub     ecx, ebx
delaylow:       loop delaylow

restore_speaker:        
	dec     edi
	jnz     getnewbyte
	

endoftalking:
	in      al, PortB
	and     al, 11111100b           ; disable speaker data and gate2
	DeviceDelay
	out     PortB, al

	call    open_timer_int
	popfd                            ; restore flags
	popad
talking_rtn:
	ret
_talking        endp
talking_        endp


talking2_        proc    near
_talking2        proc    near
;	bufferptr       equ     dword ptr [bp+8]
;	smplength       equ     dword ptr [bp+12]
;       EAX now is bufferptr
;       EDX now is smplength
;	push    ebp
;	mov     ebp, esp

	cmp     edx, 0
	jbe     talking2_rtn
	pushad
	pushfd                           ; to keep the flags
        
        push    eax
        push    edx
	call    close_timer_int

	
	in      al, PortB
	and     al, 11111100b
	mov     dh, al
	or	al, 00000010b
	mov	dl, al
	and     al, 11111100b
	DeviceDelay
	out     PortB, al
	
	mov     ebp, _adda_machinespeed
	;mov     ebp, cs:_adda_machinespeed

	cld
	pop     edi    ; smplength
	pop     esi    ; bufferptr

getnewbyte2:
	lodsb

	xor     ah, ah
	cwde
	mov     ebx, _adda_PMWtable[eax*4]

loophigh2:
	or      ebx, ebx
	jz      looplow2
	
	mov     al, dh
	out     PortB, al
	
	mov     ecx, ebx
delayhigh2:      loop delayhigh2

looplow2:
	cmp     ebx, ebp
	jnb     loophigh2b
	
	mov     al, dl
	out     PortB, al
	
	mov     ecx, ebp
	sub     ecx, ebx
delaylow2:       loop delaylow2

loophigh2b:
	or      ebx, ebx
	jz      looplow2b
	
	mov     al, dh
	out     PortB, al
	
	mov     ecx, ebx
delayhigh2b:      loop delayhigh2b

looplow2b:
	cmp     ebx, ebp
	jnb     restore_speaker2
	
	mov     al, dl
	out     PortB, al
	
	mov     ecx, ebp
	sub     ecx, ebx
delaylow2b:       loop delaylow2b

restore_speaker2:        
	dec     edi
	jnz     getnewbyte2
	

endoftalking2:
	in      al, PortB
	and     al, 11111100b           ; disable speaker data and gate2
	DeviceDelay
	out     PortB, al

	call    open_timer_int
	popfd                            ; restore flags
	popad
talking2_rtn:
	ret
_talking2        endp
talking2_        endp



cpuspeedtest_   proc    near
_cpuspeedtest   proc   near 
	
	push    ecx
	pushfd
	
	in      al, PortB
	and     al, 0fch
	DeviceDelay
	out     PortB, al
	mov     ah, al

	mov     al, 0b4h                ;  timer divider set to  maximum 65536
	out     ComReg, al
	DeviceDelay
	xor     al, al
	out     Latch2, al
	DeviceDelay
	out     Latch2, al
	DeviceDelay

	cli
	mov     ecx, CPU_LOOPS
	mov     al, ah      
	or      al, 1
	out     PortB, al               ; begin timer counter
	mov     al, 86h                 ; latch current counter

loopdelay:      loop    loopdelay       ; run null loops
	
	out     ComReg, al
	DeviceDelay
	in      al, Latch2              ; get current counter low byte
	mov     cl, al
	DeviceDelay
	in      al, Latch2              ; get current counter high byte
	mov     ch, al
	
	mov     al, ah      
	out     PortB, al
	
	xor     eax, eax
 	dec	eax      ; set the high word to FFFF
	mov     ax,  cx
	popfd
	pop     ecx
	ret
_cpuspeedtest   endp
cpuspeedtest_   endp


setspeakerdata_  proc    near
_setspeakerdata  proc    near
	pushad
	mov     esi, eax     ; sample rate
	mov     _adda_sample_rate, eax
	;mov     cs:_adda_sample_rate, eax

	call    _cpuspeedtest
	neg     eax
	mov     ebx, 255
	mul     ebx
	mov     ebx, eax                ; clockCnt*255
	
	mov     ecx, 255
	mov     edi, offset _adda_PMWtable
	mov	dword ptr [edi], 0
	;mov	dword ptr cs:[edi], 0
	add     edi, 255*4
loop_circle:
	mov     eax, CPU_LOOPS
	mul     ecx
	mov     edx, 1193180            ; Frequency of clock tick
	mul     edx	                ;   
	div     ebx                     ; ClockCountOf_CPU_LOOPS*255, 
        xor     edx, edx
        div     esi                     ; sample rate
	sub     eax, 15                 ; compensation for other instructions
	jnl     not_a_negtive
	xor     eax, eax
not_a_negtive:
	mov     [edi], eax
	;mov     cs:[edi], eax
	sub     edi, 4
	loop    loop_circle             ; calculate next value

	mov     eax, CPU_LOOPS*255
	mov     edx, 1193180            ; Frequency of clock tick
	mul     edx	                ;   
	div     ebx                     ; ClockCountOf_CPU_LOOPS*255 
        xor     edx, edx
        div     esi                     ; sample rate
	mov     _adda_machinespeed, eax
	;mov     cs:_adda_machinespeed, eax
	
	mov     _adda_windows_run, 0
	;mov     cs:_adda_windows_run, 0
	mov     ax,   01600h
	int     02fh                    ; Check for Windows installing
	cmp     al, 0
	jz      no_windows
	cmp     al, 80h
	jz      no_windows
	mov     _adda_windows_run, 1
	;mov     cs:_adda_windows_run, 1
no_windows:
	popad
	ret
_setspeakerdata endp
setspeakerdata_ endp

_TEXT	ENDS

END
